'use strict';
var callExpressionA = require('nodes');
var build = callExpressionA.build, nodes = callExpressionA.nodes;
var callExpression = require('./util/iterators');
var values = callExpression.values;
var deshorthandify = require('./transform/shorthands').default;
var arrowify = require('./transform/arrow-functions').default;
var comprehendify = require('./transform/comprehensions').default;
var forofify = require('./transform/for-of').default;
var patternify = require('./transform/patterns').default;
var defaultify = require('./transform/default-params').default;
var classify = require('./transform/classes').default;
var restify = require('./transform/rest-param').default;
var spreadify = require('./transform/spread').default;
var templateify = require('./transform/template-literals').default;
var letify = require('./transform/let-declarations').default;
var modulize = require('./transform/modules').default;
function blockify(program) {
  var statementBodies = function () {
      var comprehension = [];
      for (var iterator = values([
            '#IfStatement > alternate',
            '#IfStatement > consequent',
            '#ForStatement > body',
            '#ForInStatement > body',
            '#ForOfStatement > body',
            '#WhileStatement > body',
            '#DoWhileStatement > body'
          ])['@@iterator'](), step; !(step = iterator.next()).done;) {
        var selector = step.value;
        comprehension.push(selector + '[type!=BlockStatement]');
      }
      return comprehension;
    }();
  program.search(statementBodies).forEach(function (statement) {
    var parentNode = statement.parentNode;
    var key = parentNode.indexOf(statement);
    parentNode[key] = new nodes.BlockStatement({ body: [statement] });
  });
  program.search('#ArrowFunctionExpression[expression=true]').forEach(function (node) {
    node.expression = false;
    node.body = new nodes.BlockStatement({ body: [new nodes.ReturnStatement({ argument: node.body })] });
  });
}
function transform(tree) {
  var program = build(tree);
  blockify(program);
  modulize(program);
  deshorthandify(program);
  arrowify(program);
  comprehendify(program);
  forofify(program);
  patternify(program);
  defaultify(program);
  classify(program);
  restify(program);
  spreadify(program);
  templateify(program);
  letify(program);
  return program.toJSON();
}
exports.transform = transform;
exports.default = transform;
;